<?php
// Autor (c) Miroslav Novak, www.platiti.cz
// Pouzivani bez souhlasu autora neni povoleno
// #Ver:PRV098-49-ga9cb64a5:2022-11-14#


class CheckNonexistentFields {
	/**
	 * @param string $name
	 * @param mixed $value
	 * @return void
	 */
	function __set($name, $value ) {
		user_error("Assigning nonexistent field ".get_class($this)."::".$name);
		$this->$name = $value;
	}
}

class CheckNonexistentFieldsLogOnly {
	/**
	 * @param string $name
	 * @param mixed $value
	 * @return void
	 */
	function __set($name, $value ) {
		UniWriteErrLog(0, 'CheckNonexistentFieldsLogOnly ' . "Assigning nonexistent field ".get_class($this)."::".$name, null, null, 2);
		$this->$name = $value;
	}
}


class OrderToPayInfo extends CheckNonexistentFieldsLogOnly {
	/** @var string */
	var $shopOrderNumber;  // pouze pokud je to znamo dopredu, jinak null
	/** @var string */
	var $shopPairingInfo;
	/** @var mixed */
	var $uniAdapterData;		// serialized to blob, sessions atp
	/** @var float */
	var $amount;
	/** @var string */
	var $currency;		// ciselnik CZK, EUR, USD, GBP
	/** @var string */
	var $ccBrand;		//ciselnik dle CS pro CS, Tato polozka se jiz nepouziva
	/** @var CustomerData */
	var $customerData;   //type CustomerData
	/** @var string */
	var $language;  //ciselnik  cz, sk, en, de, pl   // ISO-639 http:www.ics.uci.edu/pub/ietf/http/related/iso639.txt (currently cs, en)
	/** @var string */
	var $description;
	/** @var string */
	var $replyUrl;
	/** @var string */
	var $notifyUrl;
	/** @var string */
	var $uniModulDirUrl;  // url adresare UniModul v rootu eshopu,    TODO: Nove uz by se melo brat jen z $this->baseConfig->uniModulDirUrl
	/** @var array<string, float> */
	var $currencyRates; //array(ISO=>rate)
	/** @var ?string */
	var $subMethod; // nazev vybrane submetody, prazdne pro defaultni metodu
	/** @var array<string, mixed> */
	var $adapterProperties; // Deprecated - aktualne se nevyuziva, asoc pole vlastnosti/pozadavku Adapteru
	/** @var int */
	var $recurrenceType; /** @see RecurrenceType enum */
	/** @var ?string */
	var $recurrenceParentShopOrderNumber; // pro recurrenceType::child je to prislusny ShopOrderNumber s recurrenceType::parent
	/** @var ?string */
	var $recurrenceDateTo; // YYYY-MM-DD
	/** @var array<int, UniCartItem> */
	var $cartItems; // array of CartItem
	/** @var ?UniCustomerStatistics */
	public $customerStatistics;
}

class CustomerData extends CheckNonexistentFieldsLogOnly {
	/** @var string */
	var $email;
	/** @var string */
	var $first_name;
	/** @var string */
	var $last_name;
	/** @var string */
	var $street;
	/** @var string */
	var $houseNumber;
	/** @var string */
	var $city;
	/** @var string */
	var $post_code;
	/** @var string */
	var $state;  // iso AZ, TX, ...http://en.wikipedia.org/wiki/ISO_3166-2:US
	/** @var string */
	var $country; // iso CZ, US, ...https://cs.wikipedia.org/wiki/ISO_3166-1
	/** @var string */
	var $phone;
	/** @var string */
	var $identifier; // identifikator zakaznika v ramci eshopu

}

class UniCartItem extends CheckNonexistentFields {
	/** @var string */
	var $name;
	/** @var float */
	var $unitPrice;   // s dani
	/** @var float */
	var $quantity;
	/** @var float */
	var $taxRate; // v procentech
	/** @var float */
	var $unitTaxAmount;   // samotna dan, dodana pozdeji, nektere adaptery ji nemusi doplnovat
	/** @var int */
	var $type; /** @see UniCartItemType enum */ // u starsich adapteru nemusi byt dosazovana
	/** @var string */
	var $productId; // u starsich adapteru nemusi byt dosazovane
}

abstract class UniCartItemType {
	const commodity = 1;
	const delivery = 2;
	const discount = 3;
}

class UniCustomerStatistics extends CheckNonexistentFields {
	/** @var ?DateTime */
	public $registeredDate;
	/** @var ?DateTime */
	public $lastOrderDate;
	/** @var ?int */
	public $orderCount;
	/** @var ?float */
	public $orderTotalSum;
	/** @var ?float */
	public $orderTotalMax;
	/** @var ?string */
	public $currency;
}

class TransactionRecord extends CheckNonexistentFieldsLogOnly {
	/** @var int */
	var $transactionPK;
	/** @var string */
	var $uniModulName;
	/** @var ?string */
	var $gwOrderNumber;
	/** @var ?string */
	var $gwPairingInfo; // druhe cislo transakce na platebni brane, typicky to, co je znamo az po vytvoreni transakce branou
	/** @var ?string */
	var $shopOrderNumber;
	/** @var string */
	var $shopPairingInfo;
	/** @var mixed */
	var $uniAdapterData;
	/** @var mixed */
	var $uniModulData;
	/** @var ?string */
	var $forexNote;
	/** @var int */
	var $orderStatus; /** @see OrderStatus enum */
	/** @var DateTimeInterface */
	var $dateCreated;
	/** @var ?DateTimeInterface */
	var $dateModified;
}

// {{{ Konfigurace

class ConfigSetting extends CheckNonexistentFieldsLogOnly {
	/** @var UniModulConfig */
	var $uniModulConfig;
	/** @var array<string, mixed> */
	var $configData;
}

class ConfigInfo extends CheckNonexistentFieldsLogOnly {
	/** @var array<int, ConfigField> */
	var $configFields;
}

class ConfigField extends CheckNonexistentFieldsLogOnly {
	/** @var string */
	var $name;
	/** @var int */
	var $type; /** @see ConfigFieldType enum */
	/** @var array<int|string, string> */
	var $choiceItems; //array(v=>l)
	/** @var string */
	var $label;
	/** @var string */
	var $comment;
}

class ConfigFieldType extends CheckNonexistentFieldsLogOnly {
	/** @var int */
	static $text = 1;
	/** @var int */
	static $choice = 2;
	/** @var int */
	static $orderStatus = 3;
	/** @var int */
	static $subMethodsSelection = 4;
	/** @var int */
	static $orderStatusMultiple = 5;
}

class RecurrenceType extends CheckNonexistentFieldsLogOnly {
	const none = 0;
	const parent = 1;
	const child = 2;
}

// }}} Konfigurace


class PrePayGWInfo extends CheckNonexistentFieldsLogOnly {
	/** @var string */
	var $paymentMethodName;
	/** @var string */
	var $paymentMethodDescription;
	/** @var string */
	var $paymentMethodIconUrl;
	/** @var bool */
	var $isPossible; // bool

	/** @var bool */
	var $selectCsPayBrand; // bool, vyber brandu pro CS
	/** @var string */
	var $selectCsPayBrandTitle; // text pro nadpis ke kart�m

	/** @var ?string */
	var $forexMessage; // string, info o automatickem prevodu

	/** @var ?string */
	var $convertToCurrency;  	// pro konverzi
	/** @var ?float */
	var $convertToCurrencyAmount;

	/** @var ?array<int, string> */
	var $subMethods;  //submetody pouzitelne pro platbu   - pokud null, tak jen hlavni modul, pokud jich je vice tak prazdny retezec znamena hlavni modul
}


class OrderReplyStatus extends CheckNonexistentFieldsLogOnly {
	/** @var ?string */
	var $shopOrderNumber;  // melo by stacit jen to pairing info, asi toto odstranit
	/** @var string */
	var $shopPairingInfo;
	/** @var ?string */
	var $gwOrderNumber;
	/** @var ?string */
	var $gwPairingInfo; // druhe cislo transakce na platebni brane, typicky to, co je znamo az po vytvoreni transakce branou
	/** @var int */
	var $orderStatus; /** @see OrderStatus enum */
	/** @var ?string */
	var $resultText; // pouziva se pri neuspechu pro predani detailni chybove hlasky
	/** @var ?string */
	var $successHtml; // pouziva se pri ok nebo pendingu pro zobrazeni instrukci k offile platbe, zobrazi se na strane s podekovanim za platbu
	/** @var ?string */
	var $forexNote;
	/** @var mixed */
	var $uniAdapterData;		// serialized from blob
}

class OrderStatus extends CheckNonexistentFieldsLogOnly {
	/** @var int */
	static $initiated = 0;
	/** @var int */
	static $successful = 1;
	/** @var int */
	static $pending = 2;
	/** @var int */
	static $failedRetriable = 3;
	/** @var int */
	static $failedFinal = 4;
	/** @var int */
	static $invalidReply = 5;  // muze byt pouzito i pro nic nerikajici stavy, aby se nezpracovali, napr PayU stav 1.
	/** @var int */
	static $gwUnpaired = 6;
}

class RedirectAction extends CheckNonexistentFieldsLogOnly {
	// bude zadne z nasledujicich tri, nebo vyplneno redirectUrl nebo redirectForm nebo inlineForm, pokud inlineForm tak totez i v redirectForm pro zpetnou kompatibilitu
	/** @var ?string */
	var $redirectUrl;
	/** @var ?string */
	var $redirectForm;
	/** @var ?string */
	var $inlineForm;
	/** @var OrderReplyStatus */
	var $orderReplyStatus;  //vysledek hned, obvykle kdyz nelze provest platbu, nebo kdyz je nutno vytvorit objednavku soucasne s presmerovanim na branu
}

class InfoBoxData extends CheckNonexistentFieldsLogOnly {
	/** @var string */
	public $title;
	/** @var string */
	public $image;
	/** @var string */
	public $link;
	/** @var string */
	public $platitiLink;
	/** @var string */
	public $platitiLinkText;
}
