<?php
// Autor (c) Miroslav Novak, www.platiti.cz
// Pouzivani bez souhlasu autora neni povoleno
// #Ver:PRV098-49-ga9cb64a5:2022-11-14#

use Automattic\WooCommerce\Utilities\NumberUtil;
use UniModul\WooCommerce\WCSession;

require_once('WCSession.php');


class WC_UniAdapter extends WC_Payment_Gateway {

	/** @var ?string */
	protected static $adapterSuccessHtml;
	/** @var ?string */
	protected static $adapterId;

	/** @var ?UniModul */
	protected $uniModul;
	/** @var ConfigInfo */
	protected $configInfo;

	/**
	 * @param ?string $uniModulName
	 * @param ?string $subMethod
	 * @return void
	 */
	public function __construct($uniModulName=null, $subMethod = null) {
		try {
			BeginUniErr();
			if ($uniModulName == null) {
				$this->id = 'uniadapter';
				EndUniErr();
				return;
			}

			$this->id = strtolower($uniModulName).'binder'.strtolower($subMethod);

			//$this->has_fields = true; // in case you need a custom credit card form

			// gateways can support subscriptions, refunds, saved payment methods,
			// but in this tutorial we begin with simple payments
			$this->supports = array(
				'products'
			);
			if(!empty($GLOBALS['UniGPWebPay_enableCapturesAndRefunds'])) {
				$this->supports[] = 'refunds';
			}


			$this->initUniModul($uniModulName, $subMethod);

			$this->method_title = $this->uniModul->getModuleSubMethodName($this->getLanguage());
			//$this->method_description  = $this->id . ' in admin description';
			$this->icon  = $this->uniModul->getModulSubMethodLogoImage2( $subMethod );

			// Method with all the options fields
			$this->init_form_fields();

			// Load the settings.
			$this->init_settings();
			$this->title = $this->get_option( 'title' );
			$this->description = $this->get_option( 'description' );
			$this->enabled = $this->get_option( 'enabled' );

			// This action hook saves the settings
			add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );

			// We need custom JavaScript to obtain a token
			//add_action( 'wp_enqueue_scripts', array( $this, 'payment_scripts' ) );

			// You can also register a webhook here
			add_action( 'woocommerce_api_'.$this->id.'_postredir', array( $this, 'webhookPostRedir' ) );
			add_action( 'woocommerce_api_'.$this->id.'_reply', array( $this, 'webhookReply' ) );
			add_action( 'woocommerce_api_'.$this->id.'_notify', array( $this, 'webhookNotify' ) );
			add_action( 'woocommerce_api_'.$this->id.'_callback', array( $this, 'webhookCallback' ) );


			add_action('woocommerce_order_status_changed', array( $this, 'woocommerce_order_status_changed' ), 10, 3);
			add_action('woocommerce_thankyou_'.$this->id, array( $this, 'showSuccessHtml'));
			add_action('woocommerce_email_before_order_table', array( $this, 'mailSuccessHtml'));
			add_filter('woocommerce_available_payment_gateways', array($this, 'isPaymentAvailable'));

			if ( WC()->session instanceof WC_Session_Handler) {
				if ( ! WC()->session->has_session() ) {
					WC()->session->set_customer_session_cookie( true );
				}
			}

			EndUniErr();
		} catch (Exception $ex) {
			ExceptionUniErr($ex);
		}
	}



	/**
	 * @param string $uniModulName
	 * @param ?string $subMethod
	 * @return void
	 */
	private function initUniModul($uniModulName, $subMethod) {
		if ($uniModulName == null || !empty($this->uniModul)) return;
		$uniFact = new UniModulFactory();
		$configInfo = $uniFact->getConfigInfo($uniModulName, $this->getLanguage(), $subMethod);
		$configSetting = $this->getConfigData($configInfo, $uniModulName);
		$this->configInfo = $configInfo;
		$this->uniModul = $uniFact->createUniModul($uniModulName, $configSetting, $subMethod);
	}

	/** @return string */
	private function getLanguage() { // TODO  WPML, Polylang...
		return substr(get_locale(), 0, 2);
	}

	/**
	 * @param ConfigInfo $configInfo
	 * @param string $uniModulName
	 * @return ConfigSetting
	 */
	protected function getConfigData($configInfo, $uniModulName) {
		$configData = array();
		$baseBinderOptions = get_option( $this->plugin_id . strtolower($uniModulName) . 'binder' . '_settings', null );
		foreach ($configInfo->configFields as $configField) {
			if ($configField->name == 'convertToCurrencyIfUnsupported') {
				$configData[$configField->name] = ""; // unused, always empty
			} else if ($configField->type === ConfigFieldType::$orderStatusMultiple) {
				$ms = isset( $baseBinderOptions[$configField->name] ) ? $baseBinderOptions[$configField->name] : array();
				if ($ms == '') $ms = array();
				$msc = array_map(function ($s) {return strpos($s, 'wc-') === 0 ? substr($s, 3) : $s;}, $ms); // remove wc- prefix
				$configData[$configField->name] = $msc;
			} else {
				$configData[$configField->name] = isset($baseBinderOptions[$configField->name]) ? $baseBinderOptions[$configField->name] : '';
			}
		}
		$uniModulConfig = new UniModulConfig();
		$uniModulConfig->mysql_server = DB_HOST;
		$uniModulConfig->mysql_dbname = DB_NAME;
		$uniModulConfig->mysql_login = DB_USER;
		$uniModulConfig->mysql_password = DB_PASSWORD;
		$uniModulConfig->uniModulDirUrl = plugins_url('WooCommerceAdapter/lib/UniModul/');
		$uniModulConfig->funcGetCallbackUrl = function ($callbackName, $arguments) {return home_url('?wc-api='.$this->id.'_callback') ."&_callbackName=".urlencode($callbackName)."&".http_build_query($arguments);};
		$uniModulConfig->adapterName = 'WooCommerce';
		$uniModulConfig->funcProcessReplyStatus = array($this, 'funcProcessReplyStatus');
		$uniModulConfig->session = new WCSession(new UniLogger());
		$cfgs = create_initialize_object('ConfigSetting', array('configData'=>$configData, 'uniModulConfig'=>$uniModulConfig));
		return $cfgs;
	}

	/**
	 * @return void
	 */
	public function init_form_fields(){
		try {
			BeginUniErr();

			$this->form_fields = array(
				'enabled' => array(
					'title'       => __( 'Enable/Disable', 'woocommerce' ),
					'type'        => 'checkbox',
					'description' => '',
					'default'     => 'no'
				),
				'title' => array(
					'title'       => __( 'Title', 'woocommerce' ),
					'type'        => 'text',
					'description' => __( 'Payment method description that the customer will see on your checkout.', 'woocommerce' ),
					'default'     => $this->uniModul->getModuleSubMethodName($this->getLanguage()),
					'desc_tip'    => true,
				),
				'description' => array(
					'title'       => __( 'Description', 'woocommerce' ),
					'type'        => 'textarea',
					'description' => __( 'Payment method description that the customer will see on your website.', 'woocommerce' ),
				),
			);
			if ($this->uniModul->subMethod == null) {
				$idfunc = function ($x) {return $x;};
				foreach ($this->configInfo->configFields as $configField) {
					$title = $this->uniModul->dictionary->get($configField->name, $this->getLanguage() );
					if ($configField->name == 'convertToCurrencyIfUnsupported') {
						// excluded
					} else if ($configField->type == ConfigFieldType::$text) {
						$this->form_fields[$configField->name] = array(
							'title'       => $title,
							'type'        => 'text',
							'sanitize_callback' => $idfunc,
						);
					} else if ($configField->type == ConfigFieldType::$choice) {
						$this->form_fields[$configField->name] = array(
							'title'       => $title,
							'type'        => 'select',
							'options'     => $configField->choiceItems,
						);
					} else if ($configField->type == ConfigFieldType::$orderStatusMultiple) {
						if ($configField->name != 'statusForRefund') {
							$statuses = wc_get_order_statuses();
						} else {
							$statuses = array();
						}
						$statuses['UNIMODUL_REFUND'] = 'Manual refund action';
						$this->form_fields[$configField->name] = array(
							'title'       => $title,
							'type'        => 'multiselect',
							'class'       => 'wc-enhanced-select',
							'options'     => $statuses,
						);
					}
				}
			}

			EndUniErr();
		} catch (Exception $ex) {
			ExceptionUniErr($ex);
		}
	}

	/**
	 * @param array<string, mixed> $available_gateways
	 * @return array<string, mixed>
	 */
	public function isPaymentAvailable( $available_gateways ) {
		try {
			BeginUniErr();

			if( is_admin() ) {
				return EndUniErr($available_gateways);
			}
			// testnout zda nas plugin je vubec v available, nez to zacit resit.
			if ( !array_key_exists($this->id,  $available_gateways) ) {
				return EndUniErr($available_gateways);
			}

			// TODO: Mozna taky cacheovat orderToPayInfo, aby se nedelalo mnohokrat pro submetody

			if( is_wc_endpoint_url( 'order-pay' ) ) { // Pay for order page
				$order_id = wc_get_order_id_by_order_key( $_GET[ 'key' ] );
				$order = wc_get_order( $order_id );
				if (is_bool($order)) throw new Exception("Order not found, key: " . $_GET[ 'key' ] . ", id: " . $order_id);
				if (!($order instanceof WC_Order)) throw new Exception("Bad order type, id: " . $order_id . ", class: " . get_class($order));
				$orderToPayInfo = $this->getOrderToPayInfo($order);
			} else if (WC()->cart != null) { // Cart/Checkout page
				$orderToPayInfo = $this->getOrderToPayInfoFromCartForPrePay(WC()->cart);
			} else {
				return EndUniErr($available_gateways); /* @phpstan-ignore-line - Nesmysl: Parameter #1 $ret of function EndUniErr expects array, array given.*/
			}
			$prePayGWInfo = $this->uniModul->queryPrePayGWInfo($orderToPayInfo);
			if (!$prePayGWInfo->isPossible) {
				unset( $available_gateways[ $this->id ] );
			}
			return EndUniErr($available_gateways);
		} catch (Exception $ex) {
			ExceptionUniErr($ex);
		}
	}

	/**
	 * @param int $order_id
	 * @return array<string, string>
	 */
	public function process_payment( $order_id ) {
		try {
			BeginUniErr();
			WC()->session->set('AdapterOrderId', $order_id);
			// Return thankyou redirect
			$ret = array(
				'result'    => 'success',
				'redirect'  => home_url('?wc-api='.$this->id.'_postredir')
			);
			return EndUniErr($ret);
		} catch (Exception $ex) {
			ExceptionUniErr($ex);
		}
	}


	/**
	 * @param string $order_id
	 * @return void
	 */
	public function webhookPostRedir($order_id) {
		try {
			BeginUniErr();
			$order_id = strval(WC()->session->get('AdapterOrderId'));
			$order = wc_get_order( $order_id );
			if (is_bool($order)) throw new Exception("Order not found, id: " . $order_id);
			if (!($order instanceof WC_Order)) throw new Exception("Bad order type, id: " . $order_id . ", class: " . get_class($order));

			$orderToPayInfo = $this->getOrderToPayInfo($order);
			$redirectAction = $this->uniModul->gatewayOrderRedirectAction($orderToPayInfo);

			if ($redirectAction->orderReplyStatus != null) {   // okamzita odpoved muze byt i zaroven s redirektem - napr. Cetelem
				$frontend_redir = $redirectAction->redirectUrl==null && $redirectAction->redirectForm==null;
				$this->processReplyStatus($redirectAction->orderReplyStatus, $frontend_redir);
			}
//				if ($redirectAction->inlineForm != null) {
//					$this->assignSmartyForAdditionalOrderForm($prePayGWInfo, $_GET['subMethod'], $redirectAction->inlineForm);
//				} else

			if ($redirectAction->redirectForm != null) {
				$this->uniModul->formRedirect($redirectAction->redirectForm);
			} else if ($redirectAction->redirectUrl != null) {
				header('Location: '.$redirectAction->redirectUrl);
				ResetUniErr();
				exit();
			}

			ResetUniErr();
			exit();

		} catch (Exception $ex) {
			ExceptionUniErr($ex);
		}
	}


	/**
	 * @param WC_Order $order
	 * @return OrderToPayInfo
	 * */
	protected function getOrderToPayInfo($order) {
		$orderToPayInfo = new OrderToPayInfo();
		$orderToPayInfo->subMethod = $this->uniModul->subMethod;
		$orderToPayInfo->shopOrderNumber = $order->get_order_number();
		$orderToPayInfo->shopPairingInfo = (string)$order->get_id();
		$orderToPayInfo->amount = $order->get_total() - (float)$order->get_total_refunded();

		$orderToPayInfo->currency = get_woocommerce_currency();
		$orderToPayInfo->language = $this->getLanguage();
		$orderToPayInfo->description = get_bloginfo();

		$customerData = new CustomerData();
		$customerData->first_name = $order->get_billing_first_name();
		$customerData->last_name = $order->get_billing_last_name();
		$customerData->email = $order->get_billing_email();
		$customerData->street = $order->get_billing_address_1().' '.$order->get_billing_address_2();
		$customerData->city = $order->get_billing_city();
		$customerData->post_code = $order->get_billing_postcode();
		$customerData->state = $order->get_billing_state();
		$customerData->country = $order->get_billing_country();
		$customerData->phone = $order->get_billing_phone();
		$customerData->identifier = (string)$order->get_customer_id();
		$orderToPayInfo->customerData = $customerData;

		$orderToPayInfo->replyUrl = home_url('?wc-api='.$this->id.'_reply');
		$orderToPayInfo->notifyUrl = home_url('?wc-api='.$this->id.'_notify');
		$orderToPayInfo->uniModulDirUrl = plugins_url('WooCommerceAdapter/lib/UniModul/');

		$orderToPayInfo->cartItems = $this->getOrderItems($order);

		return $orderToPayInfo;
	}


	/**
	 * @param WC_Abstract_Order $order
	 * @return UniCartItem[]
	*/
	private function getOrderItems($order) {
		//$order->calculate_taxes();
		//$order->recalculate_coupons();
		$cartItems = [];
		$wcRound = function($v) {return NumberUtil::round($v, wc_get_price_decimals());};
		$mkCartItem = function($item, $type) use ($wcRound) {
			$cartItem = new UniCartItem();
			$cartItem->type = $type;
			$cartItem->name = $item->get_name();
			if ($type == UniCartItemType::commodity) $cartItem->productId = $item->get_product_id();
			$cartItem->quantity = $item->get_quantity();
			$cartItem->unitPrice = $wcRound(($item->get_total('') + $item->get_total_tax('')) / $item->get_quantity());
			$cartItem->unitTaxAmount = $wcRound($item->get_total_tax() / $item->get_quantity());
			$taxes = $item->get_taxes()['total'];
			if (!empty($taxes) && $item->get_total('') != 0) {
				$cartItem->taxRate = round(100 * array_values($taxes)[0] / $item->get_total(''));
			} else {
				$cartItem->taxRate = 0;
			}
			return $cartItem;
		};

		foreach ( $order->get_items() as $item ) {
			$cartItems[] = $mkCartItem($item, UniCartItemType::commodity);
		}
		foreach ( $order->get_shipping_methods() as $item ) {
			$cartItems[] = $mkCartItem($item, UniCartItemType::delivery);
		}
		//foreach ( $order->get_coupons() as $item ) {
		//	$cartItems[] = $mkCartItem($item, UniCartItemType::discount);
		//}
		//foreach ( $order->get_fees() as $item ) {
		//	$cartItems[] = $mkCartItem($item, UniCartItemType::discount);
		//}
		return $cartItems;
	}

	/**
	 * Gets minimum orderToPayInfo for gwPrePayInfo purposes
	 * @param WC_Cart $cart
	 * @return OrderToPayInfo
	 * */
	protected function getOrderToPayInfoFromCartForPrePay($cart) {
		$orderToPayInfo = new OrderToPayInfo();
		$orderToPayInfo->subMethod = $this->uniModul->subMethod;
		$orderToPayInfo->amount = (float)$cart->get_total('edit');

		$orderToPayInfo->currency = get_woocommerce_currency();
		$orderToPayInfo->language = $this->getLanguage();
		$orderToPayInfo->description = get_bloginfo();

		$customerData = new CustomerData();
		$customerData->country = $cart->get_customer()->get_billing_country();
		$orderToPayInfo->customerData = $customerData;

		$orderToPayInfo->uniModulDirUrl = plugins_url('WooCommerceAdapter/lib/UniModul/');

		return $orderToPayInfo;
	}


	/**
	 * @return never
	 */
	public function webhookReply() {
		try {
			BeginUniErr();

			$orderReplyStatus = $this->uniModul->gatewayReceiveReply($this->getLanguage());
			$this->processReplyStatus($orderReplyStatus, true);

			EndUniErr();
			exit;
		} catch (Exception $ex) {
			ExceptionUniErr($ex);
		}
	}

	/**
	 * @return never
	 */
	public function webhookNotify() {
		try {
			BeginUniErr();

			$orderReplyStatus = $this->uniModul->gatewayReceiveNotification();
			$this->processReplyStatus($orderReplyStatus, false);

			EndUniErr();
			exit;
		} catch (Exception $ex) {
			ExceptionUniErr($ex);
		}
	}


	/**
	 * @param OrderReplyStatus $orderReplyStatus
	 * @param boolean $frontend_redir
	 * @return WC_Order vracime pro moznost zpracovani v potomcich
	 */
	protected function processReplyStatus($orderReplyStatus, $frontend_redir) {
		try {
			BeginSynchronized();
			$orderId = $orderReplyStatus->shopPairingInfo;
			$order = wc_get_order( $orderId );
			if ($order == false) {
				$orderId = preg_replace('/^pair-(.*)/', '$1', $orderId);
				$order = wc_get_order( $orderId );
			}
			if (is_bool($order)) throw new Exception("Order not found, id: " . $orderId);
			if (!($order instanceof WC_Order)) throw new Exception("Bad order type, id: " . $orderId . ", class: " . get_class($order));
			$oldStatus = $order->get_status();
			$gwOrdNumComment = $this->uniModul->dictionary->get('orderDetailGwOrdNum', $this->getLanguage()).': '.$orderReplyStatus->gwOrderNumber;
			if ($frontend_redir) WC()->session->set('AdapterSuccessHtml', $orderReplyStatus->successHtml);
			self::$adapterSuccessHtml = $orderReplyStatus->successHtml;
			self::$adapterId = $this->id;

			switch ($orderReplyStatus->orderStatus)
			{
				case OrderStatus::$successful:
					if (in_array($order->get_status(), array('pending', 'failed', 'on-hold'))) {  //on-hold zde jen pro zpetnou kompatibilitu, kdy se driv pouzival misto pending. Jinak bychom jej vubec nemeli menit, protoze jej sami nenastavujeme
						$order->payment_complete();
						$comment = $gwOrdNumComment;
						if ($orderReplyStatus->forexNote !== null) {
							$comment .= "<br/>".$this->uniModul->dictionary->get('forexNoteLabel', $this->getLanguage()). ': ' . $orderReplyStatus->forexNote;
						}
						$order->add_order_note($comment);
						$this->uniModul->logger->writeLog("INFO ADAPTER: Updating shop order number ".$orderId.", calling payment_complete, old order state ".$oldStatus.", new order state ".$order->get_status());
					} else {
						$this->uniModul->logger->writeLog("INFO ADAPTER: Not updating shop order number ".$orderId." in order status ".$oldStatus);
					}
					if ($frontend_redir) WC()->cart->empty_cart();
					$guiOk = true;
					break;
				case OrderStatus::$pending:
					if (in_array($order->get_status(), array('pending', 'failed'))) {
						if ($order->get_status() != 'pending') {
							$order->update_status( 'pending', $gwOrdNumComment);
						} else {
							$order->add_order_note($gwOrdNumComment);
						}
						$this->uniModul->logger->writeLog("INFO ADAPTER: Updating shop order number ".$orderId.", old order state ".$oldStatus.", new order state ".$order->get_status());
					} else {
						$this->uniModul->logger->writeLog("INFO ADAPTER: Not updating shop order number ".$orderId." in order status ".$oldStatus);
					}
					if ($frontend_redir) WC()->cart->empty_cart();
					$guiOk = true;
					break;
				case OrderStatus::$failedFinal:
				case OrderStatus::$failedRetriable;
					if (in_array($order->get_status(), array('pending', 'on-hold'))) {
						$order->update_status('failed', $gwOrdNumComment);
						$this->uniModul->logger->writeLog("INFO ADAPTER: Updating shop order number ".$orderId.", old order state ".$oldStatus.", new order state ".$order->get_status());
					} else {
						$this->uniModul->logger->writeLog("INFO ADAPTER: Not updating shop order number ".$orderId." in order status ".$oldStatus);
					}
				case OrderStatus::$invalidReply;
					if ($orderReplyStatus->resultText != null) {
						$order->add_order_note(__('Payment failed, please try again.', 'WooCommerceAdapter') . ' ' . $orderReplyStatus->resultText);
					}
					$guiOk = false;
					break;
				default:
					throw new Exception("Invalid orderStatus " . $orderReplyStatus->orderStatus);
			}
		} finally {
			EndSynchronized();
		}

		if ($frontend_redir) {
			if ($guiOk) {
				wp_redirect($this->get_return_url( $order ));
			} else {
				wc_add_notice( __('Payment failed, please try again.', 'WooCommerceAdapter') . ' ' . $orderReplyStatus->resultText, 'error');
				$fail_url = wc_get_endpoint_url( 'add-payment-method', '', wc_get_page_permalink( 'checkout' ) );
				wp_redirect($fail_url);
			}
		}
		return $order;
	}

	/**
	 * @param OrderReplyStatus $orderReplyStatus
	 * @return void
	 */
	public function funcProcessReplyStatus($orderReplyStatus) {
		$this->processReplyStatus($orderReplyStatus, false);
	}

	/**
	 * @return never
	 */
	public function webhookCallback() {
		try {
			BeginUniErr();

			$this->uniModul->processCallbackRequest($_GET['_callbackName'], $_GET);

			EndUniErr();
			exit;
		} catch (Exception $ex) {
			ExceptionUniErr($ex);
		}
	}


	/**
	 * @return void
	 */
	public function showProductInstallmentEmbedHtml() {
		try {
			BeginUniErr();
			if ($this->enabled == 'yes') {
				global $product;
				$price = (float)wc_get_price_including_tax($product);
				$currency = get_woocommerce_currency();
				echo $this->uniModul->ProductGetInstallmentEmbedHtml(null, $currency, $price);
			}
			EndUniErr();
		} catch (Exception $ex) {
			ExceptionUniErr($ex);
		}
	}

	/**
	 * @param int $order_id
	 * @param string $old_status
	 * @param string $new_status
	 * @return void
	 */
	public function woocommerce_order_status_changed( $order_id, $old_status, $new_status ) {
		try {
			BeginUniErr();

			$order = wc_get_order( $order_id );
			if (is_bool($order)) {
				$this->uniModul->logger->writeLog("ADAPTER WARN: woocommerce_order_status_changed Order not found, id: " . $order_id);
			} else if (!($order instanceof WC_Order)) {
				$this->uniModul->logger->writeLog("ADAPTER WARN: woocommerce_order_status_changed Bad order type, id: " . $order_id . ", class: " . get_class($order));
			} else if ( $this->id == $order->get_payment_method() ) {

				$funcOrderToPayInfo = function() use ($order) {
					return $this->getOrderToPayInfo($order);
				};
				$capture_result = $this->uniModul->shopOrderStatusChanged($order->get_order_number(), (string)$order_id, $new_status, $funcOrderToPayInfo);

				if (isset($capture_result['message'])) {
					$order->add_order_note( $capture_result['message'] );
				}
			}

			EndUniErr();
		} catch (Exception $ex) {
			ExceptionUniErr($ex);
		}
	}

	/**
	 * @param int $order_id
	 * @return void
	 */
	public function showSuccessHtml($order_id) {
		try {
			BeginUniErr();
			echo strval(WC()->session->get('AdapterSuccessHtml'));
			EndUniErr();
		} catch (Exception $ex) {
			ExceptionUniErr($ex);
		}
	}

	/** @return void  */
	public function mailSuccessHtml() {
		if ($this->id == self::$adapterId) {
			echo self::$adapterSuccessHtml;
		}
	}

	/**
	 * @return boolean|WP_Error True or false based on success, or a WP_Error object.
	 */
	public function process_refund( $order_id, $amount = null, $reason = '' ) {
		try {
			BeginUniErr();

			$order = wc_get_order( $order_id );
			if (is_bool($order)) throw new Exception("Order not found, id: " . $order_id);
			if (!($order instanceof WC_Order)) throw new Exception("Bad order type, id: " . $order_id . ", class: " . get_class($order));
			if ( $this->id == $order->get_payment_method() ) {

				$funcOrderToPayInfo = function() use ($order) {
					return EndUniErr($this->getOrderToPayInfo($order));
				};

				$refund_result = $this->uniModul->shopOrderStatusChanged($order->get_order_number(), (string)$order_id, 'UNIMODUL_REFUND', $funcOrderToPayInfo);

				if (isset($refund_result['message'])) {
					$order->add_order_note( $refund_result['message'] );
				}

				if ( isset($refund_result['success']) && $refund_result['success'] === true ) {
					return EndUniErr(true);
				}

				if (isset($refund_result['message'])) {
					return EndUniErr(new WP_Error( 'error', $refund_result['message'] ));
				}
			}
			return EndUniErr(false);

		} catch (Exception $ex) {
			ExceptionUniErr($ex);
		}
	}

}
